/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.persistence.node.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import org.openide.loaders.DataObject;
import it.tidalwave.metadata.Metadata.FindOption;
import it.tidalwave.metadata.Metadata.StoreOption;
import it.tidalwave.metadata.MetadataItemHolder;
import it.tidalwave.metadata.persistence.MetadataComposite;
import it.tidalwave.metadata.persistence.MetadataProperty;
import it.tidalwave.metadata.persistence.MetadataPropertySet;
import it.tidalwave.metadata.persistence.query.Criterion;
import it.tidalwave.metadata.persistence.spi.MetadataPersistenceSpi;
import it.tidalwave.metadata.persistence.spi.MetadataPersistenceSupport;

class MetadataPropertySetMock extends MetadataPropertySet
  {
    private final int childCount;
    private final int valueCount;

    public MetadataPropertySetMock (final Class<?> itemClass, final int childCount, final int valueCount) 
      {
        super(itemClass);
        this.childCount = childCount;
        this.valueCount = valueCount;
      }
    
    @Override
    public List<MetadataProperty> getChildProperties() 
      {
        final List<MetadataProperty> children = new ArrayList<MetadataProperty>();

        for (int i = 1; i <= childCount; i++)
          {
            final String name = getItemClass().getSimpleName() + " " + Integer.toString(i);
            children.add(new MetadataPropertyMock(this, name, 0, i * 2)); 
          }

        return children;
      }    
  }

class MetadataPropertyMock extends MetadataProperty
  {
    private final int childCount;
    private final int valueCount;
    
    public MetadataPropertyMock (final MetadataComposite parent, final String propertyName, final int childCount, final int valueCount) 
      {
        super(parent, propertyName);
        this.childCount = childCount;
        this.valueCount = valueCount;
      }
    
    @Override
    public List<MetadataProperty> getChildProperties() 
      {
        final List<MetadataProperty> children = new ArrayList<MetadataProperty>();

        for (int i = 1; i <= childCount; i++)
          {
            final String name = getItemClass().getSimpleName() + " " + Integer.toString(i);
            children.add(new MetadataPropertyMock(this, name, 0, i * 2)); 
          }

        return children;
      }    
    
    @Override
    public List<Object> getValues() 
      {
        final List<Object> values = new ArrayList<Object>();

        for (int i = 1; i <= valueCount; i++)
          {
            final Object object = (i % 2 == 0) ? (getPropertyName() + " value #" + Integer.toString(i)) : Integer.valueOf(i);
            values.add(object); 
          }

        return values;
      }      
  }

class EXIFMock
  {    
  }

class TIFFMock
  {  
  }

class IPTCMock
  {
  }

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class MetadataPersistenceMock extends MetadataPersistenceSupport implements MetadataPersistenceSpi
  {
    private final MetadataPropertySet tiff = new MetadataPropertySetMock(TIFFMock.class, 2, 0);
    private final MetadataPropertySet exif = new MetadataPropertySetMock(EXIFMock.class, 3, 0);
    private final MetadataPropertySet iptc = new MetadataPropertySetMock(IPTCMock.class, 0, 0);
    
    private final List<MetadataPropertySet> roots = Arrays.asList(tiff, exif, iptc);
    
    public List<MetadataPropertySet> getPropertySets() 
      {
        return roots;
      }

    public <T> void manage (DataObject dataObject, MetadataItemHolder<T> item, final StoreOption ... options)
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    public <T> boolean refresh (DataObject dataObject, MetadataItemHolder<T> item, final FindOption ... options)
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    public <T> void persist (DataObject dataObject, MetadataItemHolder<T> item, final StoreOption ... options)
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    public List<DataObject> findDataObjects(Criterion query) 
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }
    
    public void removeProperties (DataObject dataObject, StoreOption ... options) 
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    public List<MetadataProperty> findProperties(MetadataComposite group) 
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    public <T> List<T> findValues (MetadataProperty<T> property)
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    public int findDataObjectCount (final Criterion criterion) 
      {
        return 17;
      }
  }
