/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.persistence.node.impl;

import it.tidalwave.util.logging.Logger;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import it.tidalwave.metadata.spi.TestLoggingSupport;
import it.tidalwave.metadata.persistence.PropertyAndValue;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class MetadataRootNodeTest extends TestLoggingSupport
  {
    private static final String CLASS = MetadataRootNodeTest.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);
    
    private Node rootNode;
    
    @Before
    public void setupFixture() 
      {
        rootNode = new MetadataRootNode();
      }

    @After
    public void tearDownFixture()
      {
        rootNode = null;
      }

    @Test
    public void testGetName() 
      {
        assertEquals("All metadata", rootNode.getName());
      }

    @Test
    public void testGetDisplayName() 
      {
        assertEquals("All metadata", rootNode.getDisplayName());
      }

    @Test
    public void destroy() 
      {
      }

    @Test
    public void refreshAttributes() 
      {
      }

    @Test
    public void canCopy() 
      {
        assertFalse(rootNode.canCopy());
      }

    @Test
    public void canCut() 
      {
        assertFalse(rootNode.canCut());
      }

    @Test
    public void canDestroy() 
      {
        assertFalse(rootNode.canDestroy());
      }

    @Test
    public void canRename() 
      {
        assertFalse(rootNode.canRename());
      }
    
    @Test
    public void testGetChildren()
      throws Exception
      {
        final Children children = rootNode.getChildren();   
        assertNotNull(children);
        final Node[] nodes = getNodes(children);        
        assertEquals(3, nodes.length);
        assertNotNull(nodes[0]);
        assertNotNull(nodes[1]);
        assertNotNull(nodes[2]);
        assertEquals("EXIFMock", nodes[0].getName());
        assertEquals("IPTCMock", nodes[1].getName());
        assertEquals("TIFFMock", nodes[2].getName());
        
        final Children children1 = nodes[0].getChildren();
        final Node[] nodes1 = getNodes(children1); 
        assertEquals(3, nodes1.length);
        assertLeafNode("EXIFMock 1", nodes1[0], 2);
        assertLeafNode("EXIFMock 2", nodes1[1], 4);
        assertLeafNode("EXIFMock 3", nodes1[2], 6);
        
        final Children children2 = nodes[1].getChildren();
        final Node[] nodes2 = getNodes(children2); 
        assertEquals(0, nodes2.length);

        final Children children3 = nodes[2].getChildren();
        final Node[] nodes3 = getNodes(children3); 
        assertEquals(2, nodes3.length);
        assertLeafNode("TIFFMock 1", nodes3[0], 2);
        assertLeafNode("TIFFMock 2", nodes3[1], 4);
      }
    
    private void assertLeafNode (final String name, final Node node, final int valueCount)
      throws InterruptedException
      {
        assertNotNull(node);
        assertTrue(node instanceof MetadataCompositeNode);
        assertEquals(name, node.getName());
        final Children children = node.getChildren();
        final Node[] nodes = getNodes(children); 
        assertEquals(valueCount, nodes.length);
        
        for (int i = 0; i < valueCount; i++)
          {
            assertTrue(nodes[i] instanceof ValueNode);
            final Object object = ((i+1) % 2 == 0) ? (name + " value #" + Integer.toString(i+1)) : Integer.valueOf(i+1);
            final String expName = object.toString();
            assertEquals(expName, nodes[i].getName());
//            assertEquals(expName, nodes[i].getDisplayName()); FIXME: commented out until you remove the bind count from the display name
            final PropertyAndValue holder = nodes[i].getLookup().lookup(PropertyAndValue.class);
            assertNotNull(holder);
            assertEquals(object, holder.getValue());
          }
      }
    
    private Node[] getNodes (final Children children)
      throws InterruptedException
      {
        final long startTime = System.currentTimeMillis();
        
        Node[] nodes;
        
        for (;;)
          {
            nodes = children.getNodes();
            assertNotNull(nodes);
        
            if ((nodes.length == 1) && (nodes[0].getClass().getName().endsWith("WaitFilterNode")))
              {
                if (System.currentTimeMillis() - startTime > 2000)
                  {
                    fail("WaitFilterNode didn't go away");  
                  }
                    
                logger.fine("Waiting for wait node to disappear...");
                Thread.sleep(100);
              }
            else
              {
                break; 
              }
          }
        
        return nodes;
      }
  }