/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.persistence.node.impl;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import org.openide.nodes.Node.Property;
import org.openide.nodes.Node.PropertySet;
import it.tidalwave.metadata.persistence.MetadataProperty;
import it.tidalwave.metadata.persistence.MetadataPropertySet;
import it.tidalwave.metadata.mock.MockItem1;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class ValueNodeTest 
  {
    private MetadataPropertySet set;
    private MetadataProperty property1;
    private ValueNode valueNode;
    
    @Before
    public void setupFixture()
      {
        set = new MetadataPropertySet(MockItem1.class);
        property1 = new MetadataProperty(set, "TAG1");
        valueNode = new ValueNode(property1, "the value");
      }
    
    @After
    public void tearDownFixture()
      {
        set = null;
        property1 = null;
        valueNode = null;
      }
    
    @Test
    public void testToString()
      {
        assertEquals("ValueNode[PropertyAndValue[TAG1=the value]]", valueNode.toString());
      }
    
    @Test
    public void testPropertySets() 
      throws IllegalAccessException, InvocationTargetException, InterruptedException
      {
        final PropertySet[] propertySets = valueNode.getPropertySets();  
        assertNotNull(propertySets);
        assertEquals(1, propertySets.length);
        final PropertySet propertySet = propertySets[0];
        assertNotNull(propertySet);
        assertEquals(null, propertySet.getName());
        final Property<?>[] properties = propertySet.getProperties();
        assertNotNull(properties);
        assertEquals(1, properties.length);
        final Property<?> property = properties[0];
        assertNotNull(property);
        assertEquals("dataObjectCount", property.getName());
        assertEquals(String.class, property.getValueType());
        
        assertEquals("...", property.getValue()); // first time returns wait signal
        Thread.sleep(1000);                       // give time for the thread to compute
        assertEquals("17", property.getValue());  // actual value
      }
    
    @Test
    public void testDestroy()
      throws IOException
      {
        valueNode.destroy();
      }

    @Test
    public void testRefreshChildren() 
      {
        valueNode.refreshChildren();
      }
  }